using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Windows.Input;
using TurnaApp.Model;
using Xamarin.Forms;

namespace TurnaApp.ViewModel
{
    public class UpsertIncomeViewModel : BaseViewModel
    {

        #region Properties

        private IncomeDetail incomeDetail;

        public IncomeDetail IncomeDetail
        {
            get
            {
                return incomeDetail;
            }
            set
            {
                if (incomeDetail == value)
                {
                    return;
                }
                incomeDetail = value;
                OnPropertyChanged();
            }
        }

        private int financialPlanId;

        public int FinancialPlanId
        {
            get
            {
                return financialPlanId;
            }
            set
            {
                if (financialPlanId == value)
                {
                    return;
                }
                financialPlanId = value;
                OnPropertyChanged();
            }
        }

        private string item;

        public string Item
        {
            get
            {
                return item;
            }
            set
            {
                if (item == value)
                {
                    return;
                }
                item = value;
                OnPropertyChanged();
            }
        }

        private decimal budgetAmount;

        public decimal BudgetAmount
        {
            get
            {
                return budgetAmount;
            }
            set
            {
                if (budgetAmount == value)
                {
                    return;
                }
                budgetAmount = value;
                OnPropertyChanged();
            }
        }

        private decimal actualAmount;

        public decimal ActualAmount
        {
            get
            {
                return actualAmount;
            }
            set
            {
                if (actualAmount == value)
                {
                    return;
                }
                actualAmount = value;
                OnPropertyChanged();
            }
        }

        private DateTime? targetPayDate;

        public DateTime? TargetPayDate
        {
            get
            {
                return targetPayDate;
            }
            set
            {
                if (targetPayDate == value)
                {
                    return;
                }
                targetPayDate = value;
                OnPropertyChanged();
            }
        }

        private DateTime? actualPayDate;

        public DateTime? ActualPayDate
        {
            get
            {
                return actualPayDate;
            }
            set
            {
                if (actualPayDate == value)
                {
                    return;
                }
                actualPayDate = value;
                OnPropertyChanged();
            }
        }

        private string notes;

        public string Notes
        {
            get
            {
                return notes;
            }
            set
            {
                if (notes == value)
                {
                    return;
                }
                notes = value;
                OnPropertyChanged();
            }
        }

        private CultureInfo cultureName;

        public CultureInfo CultureName
        {
            get { return cultureName; }
            set
            {
                cultureName = value;
                OnPropertyChanged();
            }
        }

        private ObservableCollection<IncomeType> incomeType;

        public ObservableCollection<IncomeType> IncomeType
        {
            get { return incomeType; }
            set
            {
                incomeType = value;
                OnPropertyChanged("IncomeType");
            }
        }

        private IncomeType selectedIncomeType;

        public IncomeType SelectedIncomeType
        {
            get { return selectedIncomeType; }
            set
            {
                selectedIncomeType = value;
                OnPropertyChanged("SelectedIncomeType");
            }
        }

        private ObservableCollection<ItemLevel> itemLevel;

        public ObservableCollection<ItemLevel> ItemLevel
        {
            get { return itemLevel; }
            set
            {
                itemLevel = value;
                OnPropertyChanged("ItemLevel");
            }
        }

        private ItemLevel selectedItemLevel;

        public ItemLevel SelectedItemLevel
        {
            get { return selectedItemLevel; }
            set
            {
                selectedItemLevel = value;
                OnPropertyChanged("SelectedItemLevel");
            }
        }

        public Command UpsertIncomeCommand { get; set; }

        #endregion

        #region Constructor

        public UpsertIncomeViewModel(bool isUpdate, int financialPlanId, int incomeDetailId)
        {
            GetIncomeTypes();
            GetItemLevels();

            IncomeDetail = new IncomeDetail();

            if (isUpdate)
            {
                FinancialPlanId = 0;
                IncomeDetail.ID = incomeDetailId;
                GetIncomeDetail();
            }
            else
            {
                FinancialPlanId = financialPlanId;
                IncomeDetail.ID = 0;
            }

            CultureName = new CultureInfo("tr-TR");

            UpsertIncomeCommand = new Command(UpsertIncomeClicked);
        }

        #endregion

        #region Methods

        private void GetItemLevels()
        {
            ItemLevel = new ObservableCollection<ItemLevel>();

            ItemLevel.Add(new ItemLevel() { ID = 1, Name = "Yüksek" });
            ItemLevel.Add(new ItemLevel() { ID = 2, Name = "Orta" });
            ItemLevel.Add(new ItemLevel() { ID = 3, Name = "Düşük" });
        }

        private async void GetIncomeTypes()
        {
            IncomeType = new ObservableCollection<IncomeType>();

            TurnaManager turna = new TurnaManager();

            TurnaResponse response = await turna.GetIncomeTypes();

            if (response.Status)
            {
                var tempList = JsonConvert.DeserializeObject<ObservableCollection<IncomeType>>(response.Data.ToString());

                IncomeType.Clear();

                foreach (var item in tempList)
                    IncomeType.Add(item);
            }
            else
            {
                // TODO
            }

        }

        private async void GetIncomeDetail()
        {
            TurnaManager turna = new TurnaManager();

            TurnaResponse response = await turna.GetIncomeDetail(IncomeDetail.ID);

            if (response.Status)
            {
                IncomeDetail = JsonConvert.DeserializeObject<IncomeDetail>(response.Data.ToString());

                Item = IncomeDetail.Item;
                FinancialPlanId = IncomeDetail.FinancialPlan.ID;
                SelectedIncomeType = IncomeType.Where(p => p.ID == IncomeDetail.IncomeType.ID).First();
                BudgetAmount = IncomeDetail.BudgetAmount;
                ActualAmount = IncomeDetail.ActualAmount;
                TargetPayDate = IncomeDetail.TargetPayDate;
                ActualPayDate = IncomeDetail.ActualPayDate;
                SelectedItemLevel = ItemLevel.Where(p => p.ID == IncomeDetail.ItemLevel.ID).First();
                Notes = IncomeDetail.Notes;
            }
            else
            {
                await Application.Current.MainPage.DisplayAlert("Hata", response.Message + " " + response.Exception, "OK");
            }
        }

        private async void UpsertIncomeClicked(object obj)
        {
            if (SelectedIncomeType?.ID == 0 || SelectedIncomeType?.ID == null)
            {
                await Application.Current.MainPage.DisplayAlert("Uyarı", "Gelir tipi seçiniz!", "OK");
            }
            else if (Item == "" || Item == null)
            {
                await Application.Current.MainPage.DisplayAlert("Uyarı", "Gelir kaleminin adını giriniz!", "OK");
            }
            else if (BudgetAmount == 0)
            {
                await Application.Current.MainPage.DisplayAlert("Uyarı", "Bütçelenen tutarı giriniz!", "OK");
            }
            else
            {
                try
                {
                    TurnaManager turna = new TurnaManager();

                    IncomeDetail.Item = Item;
                    IncomeDetail.FinancialPlan.ID = FinancialPlanId;
                    IncomeDetail.IncomeType.ID = (object)SelectedIncomeType == null ? 0 : SelectedIncomeType.ID;
                    IncomeDetail.PaymentType.ID = 0;
                    IncomeDetail.BudgetAmount = BudgetAmount;
                    IncomeDetail.ActualAmount = ActualAmount;
                    IncomeDetail.Currency.ID = 1;
                    IncomeDetail.TargetPayDate = TargetPayDate ?? DateTime.Now;
                    IncomeDetail.ActualPayDate = ActualPayDate ?? DateTime.Now;
                    IncomeDetail.ItemLevel.ID = (object)SelectedItemLevel == null ? 0 : SelectedItemLevel.ID;
                    IncomeDetail.Notes = Notes;
                    IncomeDetail.IsDeleted = false;
                    IncomeDetail.CreateTime = DateTime.Now;
                    IncomeDetail.UpdateTime = DateTime.Now;
                    IncomeDetail.CreatedBy.ID = App.appUser.ID;
                    IncomeDetail.UpdatedBy.ID = App.appUser.ID;

                    TurnaResponse response; 

                    if (IncomeDetail.ID == 0)
                    {
                        response = await turna.InsertIncomeDetail(IncomeDetail);
                    }
                    else
                    {
                        response = await turna.UpdateIncomeDetail(IncomeDetail);
                    }

                    if (response.Status)
                    {
                        await Application.Current.MainPage.DisplayAlert("Bilgi", response.Message, "OK");
                        await Application.Current.MainPage.Navigation.PopAsync();
                    }
                    else
                    {
                        await Application.Current.MainPage.DisplayAlert("Uyarı", response.Message + " " + response.Exception, "OK");
                    }
                }
                catch (Exception ex)
                {
                    await Application.Current.MainPage.DisplayAlert("Hata", ex.Message, "OK");
                }
            }
        }

        #endregion

    }
}
